// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.13 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/JavaMap.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:21:10
//
// Module  :  JavaMap.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 267386       WASX.XD   04/18/2005   jhanders  Java Map support
// 267383       WASX.XD   04/20/2005   jhanders  touch support
// XD60_M3_B.10 WASX.XD   04/21/2005   kjlaw     javadoc cleanup.
// 284443       WASX.XD   06/22/2005   sutter    removed getIndex method from public interface
// 284334       WASX.XD   06/28/2005   sutter    removed getSession from public ObjectMap interface
// 294637.1     WASX.XD   08/11/2005   kjlaw     update javadoc for setTimeToLive.
// XD601_M3_B.5 WASX.XD   11/03/2005   cheng1    Index operations need to declare throw UnsupportedOperationException
// bug959       WASX.XD   06/13/2006   dcberg    B.4: Checkpoint replication
// b1162        XD6.1     06/23/2006   c1johnso  B.3: Query Engine requirement for getEntityMetadata()
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.ibm.websphere.objectgrid.ObjectMap.PutMode;
import com.ibm.websphere.objectgrid.plugins.io.KeySerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.ValueSerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedKey;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedValue;
import com.ibm.websphere.projector.Tuple;
import com.ibm.websphere.projector.md.EntityMetadata;

/**
 * This interface is a handle to a named Map. Maps should have homogeneous keys and values.
 * An instance of this JavaMap can only be used by the thread that is currently
 * associated with the Session that was used to get this JavaMap instance.
 * Both Session and JavaMap objects are not allowed to be shared by multiple
 * threads concurrently.
 * <p>
 * Users can get an instance of <code>JavaMap</code> from an instance of
 * <code>ObjectMap</code> by calling <code>ObjectMap.getJavaMap()</code>. There
 * are two main differences between <code>JavaMap</code> and <code>ObjectMap</code>:</p>
 * <ul>
 * <li>JavaMap extends java.util.Map. Therefore, users can cast an instance of <code>JavaMap</code>
 * to java.util.Map if they want.</li>
 * <li> The methods in <code>JavaMap</code> are defined to throw Exceptions similar to those defined
 * on the <code>java.util.Map</code> interface that is <code>ObjectGridRuntimeException</code>,
 * which is a subclass of <code>java.lang.RuntimeException</code> is used for error conditions. The
 * methods in <code>ObjectMap</code> are defined to throw <code>ObjectGridException</code>s, which are
 * checked exceptions.</li>
 * </ul>
 *
 * <p>The only methods that are supported from the <code>java.util.Map</code> interface are:
 * <ul>
 * <li>containsKey(Object)</li>
 * <li>get(Object)</li>
 * <li>put(Object, Object)</li>
 * <li>putAll(Map)</li>
 * <li>remove(Object)</li>
 * <li>clear()</li>
 * </ul>
 *
 * All other methods on the <code>java.util.Map</code> interface will throw
 * <code>java.lang.UnsupportedOperationException</code>.
 *
 * @since WAS XD 6.0, XC10
 *
 * @see ObjectMap
 * @see ObjectMap#getJavaMap()
 * @see Map
 * @see BackingMap#setCopyMode(CopyMode, Class)
 * @see BackingMap#setLockStrategy(LockStrategy)
 */
public interface JavaMap extends Map {

    /**
     * Returns the name of the JavaMap as defined by the configuration.
     *
     * @return name of JavaMap
     */
    public String getName();

    /**
     * Same as <code>get(Object)</code> method except that if pessimistic lock strategy
     * is used for this map, an upgradable lock mode is obtained for this map entry.
     * See <code>LockStrategy.PESSIMISTIC</code> for additional information.
     * Whether or not a copy of the object is returned is determined by the <code>CopyMode</code>
     * setting for this map.  See <code>CopyMode</code> for a description of each
     * possible CopyMode. If the key cannot be found in the map, a <code>null</code>
     * value will be returned. A <code>null</code> value is also returned if the value is
     * <code>null</code> and this map allows <code>null</code> values.  To distinguish the
     * two, use the <code>containsKey</code> method.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     * 
     * See {@link ObjectMap#getForUpdate(Object)} for additional specification details.
     *
     * @param key The entry to fetch
     * @return the value retrieved for update or <code>null</code>
     *
     * @throws IllegalArgumentException if key is <code>null</code>
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see #containsKey(Object)
     * @see #get(Object)
     * @see CopyMode
     * @see LockStrategy#PESSIMISTIC
     * @see ObjectMap#getForUpdate(Object)
     */
    public Object getForUpdate(Object key)
        throws ObjectGridRuntimeException;

    /**
     * Gets a list of entries from the map.
     * <p>
     * If a key in the list cannot be found, a <code>null</code>
     * value will be set at the appropriate position in the returned list.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * A return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     * 
     * See {@link ObjectMap#getAll(List)} for additional specification details.
     *
     * @param keyList A list of keys for identifying which entries to fetch
     * @return a list of values
     *
     * @throws IllegalArgumentException if keyList is null or contains a <code>null</code> element.
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see #get(Object)
     * @see ObjectMap#getAll(List)
     */
    public List getAll(List keyList) throws ObjectGridRuntimeException;

    /**
     * Same as the <code>getAll(List)</code> method except that if pessimistic lock strategy
     * is used for this map, an upgradable lock mode is obtained for these map entries.
     * See <code>LockStrategy.PESSIMISTIC</code> for additional information.
     * If a key in the list cannot be found, a <code>null</code>
     * value will be set at the appropriate position in the returned list.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * A return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     * 
     * See {@link ObjectMap#getAllForUpdate(List)} for additional specification details.
     *
     * @param keyList A list of keys for identifying which entries to fetch
     * @return a list of values
     *
     * @throws IllegalArgumentException if keyList is null or contains a <code>null</code> element.
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see #getAll(List)
     * @see #getForUpdate(Object)
     * @see LockStrategy#PESSIMISTIC
     * @see ObjectMap#getAllForUpdate(List)
     */
    public List getAllForUpdate(List keyList)
        throws ObjectGridRuntimeException;

    /**
     * Batch remove from the Map.  If a key in the list cannot be found,
     * it will be ignored.<p>
     * 
     * See {@link ObjectMap#removeAll(Collection)} for additional specification details.
     *
     * @param keyList A list of keys for identifying which entries to remove
     * @throws IllegalArgumentException if keyList is null or contains a <code>null</code> element.
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see #remove(Object)
     * @see ObjectMap#removeAll(Collection)
     */
    public void removeAll(Collection keyList)
        throws ObjectGridRuntimeException;

    /**
     * Invalidates an entry in the cache based on the key parameter.
     *
     * <p>If the key's value has changes pending in the JavaMap, it is the application's responsibility to
     * flush these changes to the Loader before invalidation.  If a flush is not performed prior to
     * invoking the invalidate operation, all pending changes for this key will be removed from the JavaMap.
     * If the key cannot be found in the map, it will be ignored.
     *
     * <p>The isGlobal parameter is used to indicate which cache level is used to invalidate the entries.
     * If isGlobal is true, when the transaction is committed, the key is removed from the
     * BackingMap also.  If a subsequent get operation is performed, the BackingMap will be skipped
     * and the Loader will be used to get the data.  If isGlobal is false, the entry is only
     * invalidated in the JavaMap (transactional cache).  If a subsequent get operation is performed,
     * the BackingMap can be used; and, if it's not in the BackingMap, the Loader will be used to
     * get the data.
     *
     * <p>A typical use of isGlobal being false is when a large number of records are touched in a
     * transaction and the application wants to evict records that are no longer used in the cache.<p>
     * 
     * See {@link ObjectMap#invalidate(Object, boolean)} for additional specification details.
     *
     * @param key Object representing the key to be used for cache entry invalidation
     * @param isGlobal Indicates whether to remove the entry from the BackingMap (true) or just the JavaMap (false).
     *
     * @throws IllegalArgumentException if key is <code>null</code>
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see ObjectMap#invalidate(Object, boolean)
     */
    public void invalidate(Object key, boolean isGlobal)
        throws ObjectGridRuntimeException;

    /**
     * Invalidate a set of cache entries based on the Collection of
     * keys provided.  If a key in the collection cannot be found, it will be ignored.<p>
     * 
     * See {@link ObjectMap#invalidateAll(Collection, boolean)} for additional specification details.
     *
     * @param keyList A Collection of keys representing the entries to be invalidated
     * @param isGlobal Indicates whether to remove the entry from the BackingMap (true) or just the JavaMap (false).
     *
     * @throws IllegalArgumentException if keyList is null or contains a <code>null</code> element.
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see #invalidate(Object, boolean)
     * @see ObjectMap#invalidateAll(Collection, boolean)
     */
    public void invalidateAll(Collection keyList, boolean isGlobal)
        throws ObjectGridRuntimeException;

    /**
     * Establishes the number of seconds that any given cache entry can live for, which is
     * refered to as "time to live" or TTL.  Setting a new TTL value affects cache entries
     * that are accessed after this method call occurs. It does not affect any cache entry
     * that was created or accessed prior to this method call.  By calling this method on
     * this <code>JavaMap</code>, any previous value set by the <code>BackingMap.setTimeToLive(int)</code>
     * method is overridden for this JavaMap.  If this method is never called on the JavaMap,
     * the TTL value from the BackingMap setting is used by default.  If TTL is never set on
     * the BackingMap, the cache entry can live "forever".
     * <p>
     * This method can only be used when the <code>TTLType</code> is set to <code>LAST_ACCESS_TIME</code>
     * on the BackingMap.  If this method is called on the JavaMap and the <code>TTLType</code>
     * is something other than <code>LAST_ACCESS_TIME</code>, an IllegalStateException is thrown.
     *
     * @param  ttl   is the time-to-live value in seconds.  The value must be >= 0. A value of 0
     *               is used to indicate the cache entry can live "forever".  Use of the
     *               constant <code>ObjectMap.TTL_FOREVER</code> is recommended when "forever" is desired.
     *
     * @return previous time-to-live value in seconds. The constant <code>ObjectMap.TTL_FOREVER</code> can be used
     *                  to determine if the previous TTL was set to "forever".
     *
     * @throws IllegalArgumentException if seconds argument is < 0.
     * @throws IllegalStateException if <code>BackingMap.getTtlEvictorType()</code> returns anything other
     *         than <code>TTLType.LAST_ACCESS_TIME</code>.
     *
     * @see BackingMap#setTimeToLive(int)
     * @see ObjectMap#setTimeToLive(int)
     * @see TTLType#LAST_ACCESS_TIME
     */
    public int setTimeToLive(int ttl);

    /**
     * Performs an explicit update of a given entry.<p>
     * 
     * A <code>get</code> operation is not required prior to invoking the
     * <code>update</code> method (unlike the <code>put</code> method).
     * Also, an <code>update</code> invocation will never insert a new record.
     * If a the map's <code>LockStrategy</code> is <code>LockStrategy.OPTIMISTIC</code>
     * this method will implicitly get the entry so as to have the version value
     * of the object for when this method was invoked. Whether or not a copy of the object is
     * made when transaction is committed is determined by the CopyMode setting for this map.
     * See <code>CopyMode</code> for a description of each possible CopyMode.<p>
     *
     * If a key cannot be found in the map during commit, a TransactionException will be thrown.<p>
     * 
     * See {@link ObjectMap#update(Object, Object)} for additional specification details.
     *
     * @param key Identifies the entry to be updated
     * @param value The updated value for this entry
     * @throws IllegalArgumentException if key is <code>null</code> or if the map does not
     *         allow <code>null</code> values and value is <code>null</code>.
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see #insert(Object, Object)
     * @see #put(Object, Object)
     * @see CopyMode
     * @see LockStrategy#OPTIMISTIC
     * @see ObjectMap#update(Object, Object)
     */
    public void update(Object key, Object value)
        throws ObjectGridRuntimeException;

    /**
     * Performs an explicit insert of a given entry.
     * <p>
     * The key must not exist before
     * executing this method.  Also, an <code>insert</code> invocation will never update an
     * existing record.  Whether or not a copy of the object is made when a transaction is committed
     * is determined by the CopyMode setting for this map.  See <code>CopyMode</code> for a description
     * of each possible CopyMode.<p>
     *
     * If the key is already in the map, a TransactionException will be thrown during commit.<p>
     * 
     * See {@link ObjectMap#insert(Object, Object)} for additional specification details.
     *
     * @param key Identifies the entry to be inserted
     * @param value The value for this entry
     *
     * @throws IllegalArgumentException if key is <code>null</code> or if the map does not
     *         allow <code>null</code> values and value is <code>null</code>.
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see #put(Object, Object)
     * @see #update(Object, Object)
     * @see CopyMode
     * @see ObjectMap#insert(Object, Object)
     */
    public void insert(Object key, Object value)
        throws ObjectGridRuntimeException;

    /**
     * Returns a reference to the named index that can be used with this Map. This index
     * cannot be shared between threads and works on the same rules as <code>Session</code>.
     * The returned value should be cast to the right index interface such as <code>MapIndex</code>,
     * <code>MapRangeIndex</code> or a custom index interface such as a geo spatial index.
     *
     * @param name The index name
     *
     * @return A reference to the index, it must be cast to the appropriate index interface.
     *
     * @throws IndexUndefinedException if the index is not defined on the BackingMap
     * @throws IndexNotReadyException if the index is a dynamic index and it is not ready
     * @throws UnsupportedOperationException if the map is a distributed map
     *
     * @since WAS XD 6.0.1
     */
    public Object getIndex(String name)
        throws IndexUndefinedException, IndexNotReadyException, UnsupportedOperationException;

    /**
     * Pushes the current set of changes for the <code>JavaMap</code>
     * instance to the <code>Loader</code> without committing the changes.  The
     * changes are not propagated to the <code>BackingMap</code> either.  This is
     * useful for re-priming the <code>Loader</code>'s data without committing
     * the current transaction and starting over.
     *
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see Session#flush()
     * @see ObjectMap#flush()
     */
    public void flush() throws ObjectGridRuntimeException;

    /**
     * This method of the <code>java.util.Map</code> interface is not supported.
     *
     * @throws UnsupportedOperationException indicating this method is not supported
     *
     * @return the number of key-value mappings in this map.
     */
    public int size();

    /**
     * This method of the <code>java.util.Map</code> interface is not supported.
     *
     * @throws UnsupportedOperationException indicating this method is not supported
     *
     * @return <code>true</code> if this map contains no key-value mappings.
     */
    public boolean isEmpty();

    /**
     * Returns <code>true</code> if this map contains a mapping for the specified
     * key.  ObjectGrid does not support null keys. If you configured the map
     * to support <code>null</code> values, this method can be used to determine whether a
     * key is contained in the map or not.<p>
     *
     * This API does not hold any locks when using pessimistic locking.<p>
     * 
     * See {@link ObjectMap#containsKey(Object)} for additional specification details.
     *
     * @param key key whose presence in this map is to be tested.
     * @return <code>true</code> if this map contains a mapping for the specified
     *         key.
     *
     * @throws IllegalArgumentException if <code>null</code> key parameter is passed in
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see ObjectMap#containsKey(Object)
     */
    public boolean containsKey(Object key);

    /**
     * This method of the <code>java.util.Map</code> interface is not supported.
     *
     * @param value value whose presence in this map is to be tested.
     *
     * @throws UnsupportedOperationException indicating this method is not supported
     *
     * @return <code>true</code> if this map maps one or more keys to the
     *         specified value.
     */
    public boolean containsValue(Object value);

    /**
     * Retrieves the object from the cache at the given key.
     * <p>
     * Whether or not a copy of the object is returned is determined by the CopyMode
     * setting for this map.  See <code>CopyMode</code> for a description of each
     * possible CopyMode. If the key cannot be found in the map, a <code>null</code>
     * value will be returned. A <code>null</code> value is also returned if a value is
     * <code>null</code> and this map allows <code>null</code> values.  To distinguish the
     * two, use the <code>containsKey</code> method.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     * 
     * See {@link ObjectMap#get(Object)} for additional specification details.
     *
     * @param key The entry to fetch
     * @return the value, <code>null</code>, {@link SerializedValue} or {@link Tuple}
     *
     * @throws IllegalArgumentException if key is <code>null</code>
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see #containsKey(Object)
     * @see #getForUpdate(Object)
     * @see CopyMode
     * @see ObjectMap#get(Object)
     */
    public Object get(Object key);

    /**
     * Puts the Object value into the cache at location represented by key.<p>
     * 
     * The values will be pushed down to the BackingMap/Loader at commit time and  
     * has two behaviors, which can be altered using the {@link ObjectMap#setPutMode(PutMode)} property:
     * <table>
     * <tr><td valign="top">{@link PutMode#INSERTUPDATE}</td><td>(Deprecated)  A
     * put without a preceding get is an insert. For an entry in a map, a put following a get is always an update.
     * However, if the entry is not in the map, a put following a get is an insert.</td></tr>
     * <tr><td valign="top">{@link PutMode#UPSERT}</td><td>The values are put into the map using the specification of
     * the {@link ObjectMap#upsertAll(LinkedHashMap)}.</td></tr>
     * </table>
     *
     * <p>Whether or not a copy of the object is made when transaction is committed
     * is determined by the copy mode setting for this map.  See <code>CopyMode</code>
     * for a description of each possible copy mode.</p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     * 
     * See {@link ObjectMap#put(Object, Object)} for additional specification details.
     *
     * @param key The entry to put into the map
     * @param value The value to put into the map using the key
     * @return If {@link PutMode#INSERTUPDATE} is set, return the previous value in this transaction.  
     * If {@link PutMode#UPSERT} is set, the return value is null.
     *
     * @throws IllegalArgumentException if key is <code>null</code>, or if the map does not
     *         allow <code>null</code> values and value is <code>null</code>
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see CopyMode
     * @see ObjectMap#put(Object, Object)
     */
    public Object put(Object key, Object value);

    /**
     * Removes the Object value from the cache represented by key.
     * <p>
     * This removal will be pushed down to the BackingMap/Loader at commit time.
     * If the key cannot be found in the map, a <code>null</code> value will be returned.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     * 
     * See {@link ObjectMap#remove(Object)} for additional specification details.
     *
     * @param key The entry to remove
     * @return the current value at invocation time
     *
     * @throws IllegalArgumentException if key is <code>null</code>
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see ObjectMap#remove(Object)
     */
    public Object remove(Object key);

    /**
     * Puts each of the Object values into the cache at location represented by the corresponding key contained in the Map.<p>
     * 
     * The value will be pushed down to the BackingMap/Loader at commit time and  
     * has two behaviors, which can be altered using the {@link ObjectMap#setPutMode(PutMode)} property:
     * <table>
     * <tr><td valign="top">{@link PutMode#INSERTUPDATE}</td><td>(Deprecated)  A
     * put without a preceding get is an insert. For an entry in a map, a put following a get is always an update.
     * However, if the entry is not in the map, a put following a get is an insert.</td></tr>
     * <tr><td valign="top">{@link PutMode#UPSERT}</td><td>The value is put into the map using the specification of
     * the {@link ObjectMap#upsert(Object, Object)}.</td></tr>
     * </table><p>
     *
     * Whether or not a copy of the objects contained in the map is made when transaction is committed
     * is determined by the copy mode setting for this map.  See <code>CopyMode</code>
     * for a description of each possible copy mode.<p>
     *
     * An existing Map object will be passed in to use for obtaining the keys
     * and values to be inserted or updated into the existing Map.<p>
     * 
     * See {@link ObjectMap#putAll(Map)} for additional specification details.
     *
     * @param map The key/values to be put into the map.
     *
     * @throws IllegalArgumentException if map is <code>null</code> or contains a <code>null</code> key or if <code>null</code>
     *                              values are not allowed and map contains a <code>null</code> value.
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see #put(Object, Object)
     * @see ObjectMap#putAll(Map)
     */
    public void putAll(Map map);

    /**
     * Clear all keys from the Map.
     * <p>
     * This method is an auto-commit call, so a session should not be explicitly
     * begun or committed when calling clear on the Map.
     *
     * @throws ObjectGridRuntimeException if an error occurs during processing
     * @since WAS XD 6.1.0.3
     */
    public void clear();

    /**
     * This method of the <code>java.util.Map</code> interface is not supported.
     *
     * @throws UnsupportedOperationException indicating this method is not supported
     *
     * @return a set view of the keys contained in this map.
     */
    public Set keySet();

    /**
     * This method of the <code>java.util.Map</code> interface is not supported.
     *
     * @throws UnsupportedOperationException indicating this method is not supported
     *
     * @return a collection view of the values contained in this map.
     */
    public Collection values();

    /**
     * This method of the <code>java.util.Map</code> interface is not supported.
     *
     * @throws UnsupportedOperationException indicating this method is not supported
     *
     * @return a set view of the mappings contained in this map.
     */
    public Set entrySet();

    /**
     * Updates the last access time in the BackingMap without retrieving
     * the value to the JavaMap.
     * <p>
     * The last access time is updated during
     * commit.  If the key does not exist in the BackingMap, a
     * TransactionException will be returned during commit processing.<p>
     * 
     * See {@link ObjectMap#touch(Object)} for additional specification details.
     *
     * @param key key to be touched
     *
     * @throws IllegalArgumentException if key is <code>null</code>
     * @throws ObjectGridRuntimeException if an error occurs during processing
     *
     * @see ObjectMap#touch(Object)
     */
    void touch(Object key);

    /**
     * Allows the CopyMode for the Map to be overridden on this map on this session only.
     * <p>
     * This method allows an application to use an optimal CopyMode TRANSACTION by
     * TRANSACTION as its needs dictate. The CopyMode cannot be changed
     * during a transaction. There must be no active transaction when this method is called.
     *
     * @param copyMode must be one of the final static variables defined in <code>CopyMode</code>.
     *        See <code>CopyMode</code> class for an explanation of each mode and how the valueInterface
     *        is used for <code>CopyMode.COPY_ON_WRITE</code> .
     * @param valueInterface the value interface Class object.  Specify null in version 7.1 and later.
     *
     * @throws IllegalArgumentException if copyMode is <code>null</code> or COPY_ON_WRITE CopyMode is
     *                                  specified and the required value interface parameter is <code>null</code>
     * @throws ObjectGridRuntimeException if a transaction is active on the associated session or
     *                                    an error occurs during processing
     *
     * @see BackingMap#setCopyMode(CopyMode, Class)
     * @see CopyMode
     * @see ObjectMap#setCopyMode(CopyMode, Class)
     */
    public void setCopyMode(CopyMode copyMode, Class valueInterface)
        throws ObjectGridRuntimeException;

    /**
     * Resets the CopyMode back to the one in the BackingMap.
     * <p>
     * This method is used to reverse a previous setCopyMode method call for this JavaMap.
     * This method can only be called when no transaction is active on the associated session.
     *
     * @throws ObjectGridRuntimeException if a transaction is active on the associated session
     *
     * @see #setCopyMode(CopyMode, Class)
     * @see ObjectMap#clearCopyMode()
     */
    public void clearCopyMode()
        throws ObjectGridRuntimeException;

    /**
     * Retrieves a key off the map in first-in-first-out (FIFO) insert order. The entry is locked by the session such that other calls to getNextKey
     * will not return the same key. The key can be used to remove or manipulate the value although leaving the entry will result in the key remaining
     * at the beginning of the queue. This order is optimized for performance and is not guaranteed especially across partitions or in highly
     * concurrent environments.<p>
     * 
     * The return value is a {@link SerializedKey} when {@linkplain OutputFormat#RAW} is set for the keys.
     * The default key output format for maps that are associated with a {@linkplain KeySerializerPlugin} is {@linkplain OutputFormat#RAW}.
     * The SerializedKey allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     * 
     * See {@link ObjectMap#getNextKey(long)} for additional specification details.
     *
     * @param timeout
     *            The period of time to wait for an entry to become available on the queue.
     * @return The next available key in the map.
     * @see ObjectMap#getNextKey(long)
     */
    public Object getNextKey(long timeout);

    /**
     * Retrieve the metadata for the entity associated with this map.
     * @return the EntityMetadata if an entity is associated with this map or null if there is no
     * entity associated with this map.
     * @since WAS XD 6.1
     */
    public EntityMetadata getEntityMetadata();
}