// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ObjectMap.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// XD60_M2_B.2  WASX.XD   03/24/2005   kjlaw     cleanup javadoc of add/removeMapEventListener methods.
// d263795      WASX.XD   04/05/2005   cheng1    exception handling
// XD60_M3_B.10 WASX.XD   04/07/2005   sutter    Re-factor datagrid/datagrid.impl
// 267386       WASX.XD   04/18/2005   jhanders  Java Map support
// 267383       WASX.XD   04/20/2005   jhanders  touch support
// XD60_M3_B.10 WASX.XD   04/21/2005   kjlaw     javadoc cleanup.
// 284443       WASX.XD   06/22/2005   sutter    removed getIndex method from public interface
// 284334       WASX.XD   06/28/2005   sutter    removed getSession from public ObjectMap interface
// 294637.1     WASX.XD   08/11/2005   kjlaw     update javadoc for setTimeToLive.
// XD601_M1_B.5 WASX.XD   08/26/2005   cheng1    add dynamic indexing support
// XD601_M3_B.5 WASX.XD   11/03/2005   cheng1    Index operations need to declare throw UnsupportedOperationException
// d344167      WXD601.XD 02/03/2006   kjlaw     correct behavior of TTL when setTimeToLive is never called.
// b1436        XD6.1     06/10/2006   jhanders  update javadoc for getNextKey
// b1162        XD6.1     06/23/2006   c1johnso  B.3: Query Engine requirement for getEntityMetadata()
// b10896       6.1.0.1   12/14/2007   cheng1    Index / query forUpdate intent support
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid;

import java.security.AccessControlException;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.ibm.websphere.objectgrid.datagrid.AgentManager;
import com.ibm.websphere.objectgrid.plugins.LogElement;
import com.ibm.websphere.objectgrid.plugins.io.KeyDataSerializer;
import com.ibm.websphere.objectgrid.plugins.io.KeySerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.ValueDataSerializer;
import com.ibm.websphere.objectgrid.plugins.io.ValueSerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedKey;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedValue;
import com.ibm.websphere.objectgrid.security.MapPermission;
import com.ibm.websphere.projector.Tuple;
import com.ibm.websphere.projector.md.EntityMetadata;

/**
 * This is a handle to a named Map. Maps should have homogeneous keys and values. An instance of this ObjectMap can only
 * be used by the thread that is currently associated with the Session that was used to get this ObjectMap instance.
 * Both Session and ObjectMap objects are not allowed to be shared by multiple threads concurrently. 
 *
 * <p>
 * The <code>CopyMode</code> setting on the map determines whether or not a copy of the value is returned by get
 * methods. It also determines whether or not a copy of the committed value is made at commit time. The
 * <code>LockStrategy</code> setting for the map determines whether or not a lock is obtained for each map entry
 * accessed by the transaction, the lock mode of the lock obtained, and when the lock is obtained.<p>
 *
 * Each data access method includes a "Specification details" table that includes the following information:
 * <table>
 * <tr><td valign="top"><strong>Required permission:</strong></td><td valign="top">The permission required to use the API.</td></tr>
 * <tr><td valign="top"><strong>Pessimistic read lock acquired:</strong></td><td valign="top">The type of lock that is acquired when using pessimistic
 * locking with repeatable read transaction isolation.</td></tr>
 * <tr><td valign="top"><strong>Pessimistic read lock held:</strong></td><td valign="top">The type of lock that is held for the duration of the transaction
 *  when using pessimistic locking with repeatable read transaction isolation.  Locks can be upgraded but not demoted during a transaction.</td></tr>
 * <tr><td valign="top"><strong>Transaction:</strong></td><td valign="top">The state of the transaction when invoking the API.
 * <ul>
 * <li><strong>Manual</strong> - The transaction is explicitly demarcated using the associated {@link Session} and the
 * specified API is used within the scope of that transaction.</li>
 * <li><strong>Automatic</strong> - The transaction is automatically demarcated.  This is also referred to as an auto-commit
 * transaction or API which the associated {@link Session#isTransactionActive()} is false when invoking the API.</li>
 * </ul></td></tr>
 * <tr><td valign="top"><strong>Cache tier:</strong></td><td valign="top">
 * Identifies the map cache tiers that are included when fetching or updating cache entries in the call and under what circumstances.<br> 
 * The following tiers are available for client maps:
 * <ul>
 * <li>Transactional Cache
 * <li>Near Cache (if enabled)
 * <li>Server Cache
 * <li>Loader (if enabled)
 * </ul>
 * The following tiers are available for local maps:
 * <ul>
 * <li>Transactional Cache
 * <li>Local Cache
 * <li>Loader (if enabled)
 * </ul>
 * </td></tr>
 * </table>
 *
 * @since WAS XD 6.0, XC10
 *
 * @see Session#getMap(String)
 * @see BackingMap#setCopyMode(CopyMode, Class)
 * @see BackingMap#setLockStrategy(LockStrategy)
 */
public interface ObjectMap {

    /**
     * Returns the name of the ObjectMap as defined by the configuration.
     *
     * @return name of ObjectMap
     */
    String getName();

    /**
     * Retrieves the object from the cache at the given key.
     * <p>
     * Whether or not a copy of the object is returned is determined by the CopyMode setting for this map. See
     * <code>CopyMode</code> for a description of each possible CopyMode. If the key cannot be found in the map, a
     * <code>null</code> value will be returned. A <code>null</code> value is also returned if a value is
     * <code>null</code> and this map allows <code>null</code> values. To distinguish the two, use the
     * <code>containsKey</code> method.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#READ}</td></tr>
     * <tr><td valign="top">Pessimistic read lock acquired:</td><td valign="top">{@link LockMode#SHARED}</td></tr>
     * <tr><td valign="top">Pessimistic read lock held:</td><td valign="top">Yes</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Progresses to all tiers until the key is found.</td></tr>
     * </table>
     *
     * @param key
     *            The entry to fetch
     * @return the value, <code>null</code>, {@link SerializedValue} or {@link Tuple}
     *
     * @throws IllegalArgumentException
     *             if key is <code>null</code>
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #containsKey(Object)
     * @see #getForUpdate(Object)
     * @see CopyMode
     */
    Object get(Object key) throws ObjectGridException;

    /**
     * Puts the Object value into the cache at location represented by key.<p>
     * 
     * The value will be pushed down to the BackingMap/Loader at commit time and  
     * has two behaviors, which can be altered using the {@link #setPutMode(PutMode)} property:
     * <table>
     * <tr><td valign="top">{@link PutMode#INSERTUPDATE}</td><td>(Deprecated)  A
     * put without a preceding get is an insert. For an entry in a map, a put following a get is always an update.
     * However, if the entry is not in the map, a put following a get is an insert.</td></tr>
     * <tr><td valign="top">{@link PutMode#UPSERT}</td><td>The value is put into the map using the specification of
     * the {@link #upsert(Object, Object)}.</td></tr>
     * </table><p>
     *
     * Whether or not a copy of the object is made when transaction is committed is determined by the copy mode setting
     * for this map. See <code>CopyMode</code> for a description of each possible copy mode.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#WRITE}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit. 
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.</td></tr>
     * </table>
     *
     * @param key
     *            The entry to put into the map
     * @param value
     *            The value to put into the map using the key
     * @return If {@link PutMode#INSERTUPDATE} is set, return the previous value in this transaction.  If {@link PutMode#UPSERT} is set, 
     * the return value is null.
     *
     * @throws IllegalArgumentException
     *             if key is <code>null</code>, or if the map does not allow <code>null</code> values and value is
     *             <code>null</code>
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     * @see CopyMode
     */
    Object put(Object key, Object value) throws ObjectGridException;

    /**
     * Same as <code>get(Object)</code> method except that if pessimistic lock strategy is used for this map, an
     * upgradable lock mode is obtained for this map entry. See <code>LockStrategy.PESSIMISTIC</code> for additional
     * information. Whether or not a copy of the object is returned is determined by the <code>CopyMode</code> setting
     * for this map. See <code>CopyMode</code> for a description of each possible CopyMode. If the key cannot be found
     * in the map, a <code>null</code> value will be returned. A <code>null</code> value is also returned if the
     * value is <code>null</code> and this map allows <code>null</code> values. To distinguish the two, use the
     * <code>containsKey</code> method.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#READ}</td></tr>
     * <tr><td valign="top">Pessimistic read lock acquired:</td><td valign="top">{@link LockMode#UPGRADABLE}<br>Note:  Transaction isolation is ignored.</td></tr>
     * <tr><td valign="top">Pessimistic locks held:</td><td valign="top">Yes</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Progresses to all tiers until the key is found and the appropriate lock is acquired.</td></tr>
     * </table>
     *
     * @param key
     *            The entry to fetch
     * @return the value, <code>null</code>, {@link SerializedValue} or {@link Tuple}
     *
     * @throws IllegalArgumentException
     *             if key is <code>null</code>
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #containsKey(Object)
     * @see #get(Object)
     * @see CopyMode
     * @see LockStrategy#PESSIMISTIC
     */
    Object getForUpdate(Object key) throws ObjectGridException;

    /**
     * Removes the Object value from the cache represented by key.
     * <p>
     * This removal will be pushed down to the BackingMap/Loader at commit time. If the key cannot be found in the map,
     * a null value will be returned.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#REMOVE}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit. 
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.</td></tr>
     * </table>
     *
     * @param key
     *            The entry to remove
     * @return the current value at invocation time
     *
     * @throws IllegalArgumentException
     *             if key is <code>null</code>
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     */
    Object remove(Object key) throws ObjectGridException;

    /**
     * Gets a list of entries from the map.
     * <p>
     * If a key in the list cannot be found, a <code>null</code> value will be set at the appropriate position in the
     * returned list.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * A return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#READ}</td></tr>
     * <tr><td valign="top">Pessimistic read lock acquired:</td><td valign="top">{@link LockMode#SHARED}</td></tr>
     * <tr><td valign="top">Pessimistic read lock held:</td><td valign="top">Yes</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">For each key, progresses to all tiers until the key is found.</td></tr>
     * </table>
     *
     * @param keyList
     *            A list of keys for identifying which entries to fetch
     * @return a list of values
     *
     * @throws IllegalArgumentException
     *             if keyList is null or contains a <code>null</code> element.
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #get(Object)
     */
    List/* <Object> */getAll(List/* <Object key> */keyList) throws ObjectGridException;

    /**
     * Same as the <code>getAll(List)</code> method except that if pessimistic lock strategy is used for this map, an
     * upgradable lock mode is obtained for these map entries. See <code>LockStrategy.PESSIMISTIC</code> for
     * additional information. If a key in the list cannot be found, a <code>null</code> value will be set at the
     * appropriate position in the returned list.<p>
     * 
     * The return value is a {@link SerializedValue} when using the {@link CopyMode#COPY_TO_BYTES_RAW} CopyMode or {@link OutputFormat#RAW}
     * OutputFormat with a {@linkplain ValueSerializerPlugin} plug-in defined on the {@linkplain BackingMap}.  The SerializedValue 
     * allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * A return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#READ}</td></tr>
     * <tr><td valign="top">Pessimistic read lock acquired:</td><td valign="top">{@link LockMode#UPGRADABLE}<br>Note:  Transaction isolation is ignored.</td></tr>
     * <tr><td valign="top">Pessimistic locks held:</td><td valign="top">Yes</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Progresses to all tiers until the keys are found and the appropriate locks are acquired.</td></tr>
     * </table>
     *
     * @param keyList
     *            A list of keys for identifying which entries to fetch
     * @return a list of values
     *
     * @throws IllegalArgumentException
     *             if keyList is null or contains a <code>null</code> element.
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #getAll(List)
     * @see #getForUpdate(Object)
     * @see LockStrategy#PESSIMISTIC
     */
    List/* <Object> */getAllForUpdate(List/* <Object key> */keyList) throws ObjectGridException;

    /**
     * Batch remove from the Map. If a key in the list cannot be found, it will be ignored.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#REMOVE}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit. 
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.</td></tr>
     * </table>
     *
     * @param keyList
     *            A list of keys for identifying which entries to remove
     * @throws IllegalArgumentException
     *             if keyList is null or contains a <code>null</code> element.
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #remove(Object)
     */
    void removeAll(Collection/* <Object key> */keyList) throws ObjectGridException;

    /**
     * Puts each of the Object value into the cache at location represented by key contained in the Map.
     * <p>
     * 
     * The values will be pushed down to the BackingMap/Loader at commit time and  
     * has two behaviors, which can be altered using the {@link #setPutMode(PutMode)} property:
     * <table>
     * <tr><td valign="top">{@link PutMode#INSERTUPDATE}</td><td>(Deprecated)  A
     * put without a preceding get is an insert. For an entry in a map, a put following a get is always an update.
     * However, if the entry is not in the map, a put following a get is an insert.</td></tr>
     * <tr><td valign="top">{@link PutMode#UPSERT}</td><td>The values are put into the map using the specification of
     * the {@link #upsertAll(LinkedHashMap)}.</td></tr>
     * </table><p>
     *
     * <p>
     * Whether or not a copy of the object is made when transaction is committed is determined by the copy mode setting
     * for this map. See <code>CopyMode</code> for a description of each possible copy mode.
     * </p>
     *
     * <p>
     * An existing Map object will be passed in to use for obtaining the keys and values to be inserted or updated into
     * the existing Map.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#WRITE}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit. 
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.</td></tr>
     * </table>
     *
     * @param map
     *            The key/values to be put into the map.
     *
     * @throws IllegalArgumentException
     *             if map is <code>null</code> or contains a <code>null</code> key or if <code>null</code> values
     *             are not allowed and map contains a <code>null</code> value.
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #put(Object, Object)
     */
    void putAll(Map map) throws ObjectGridException;

    /**
     * Invalidates an entry in the cache based on the key parameter.
     *
     * <p>
     * If the key's value has changes pending in the ObjectMap, it is the application's responsibility to flush these
     * changes to the Loader before invalidation. If a flush is not performed prior to invoking the invalidate
     * operation, all pending changes for this key will be removed from the ObjectMap. If the key cannot be found in the
     * map, it will be ignored.
     *
     * <p>
     * The isGlobal parameter is used to indicate which cache level is used to invalidate the entries. If isGlobal is
     * true, when the transaction is committed, the key is removed from the BackingMap also. If a subsequent get
     * operation is performed, the BackingMap will be skipped and the Loader will be used to get the data. If isGlobal
     * is false, the entry is only invalidated in the ObjectMap (transactional cache). If a subsequent get operation is
     * performed, the BackingMap can be used; and, if it's not in the BackingMap, the Loader will be used to get the
     * data.
     *
     * <p>
     * A typical use of isGlobal being false is when a large number of records are touched in a transaction and the
     * application wants to evict records that are no longer used in the cache.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#INVALIDATE}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit except the Loader.
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.  
     * Set the <code>isGlobal</code> parameter to <code>false</code> to limit the operation to the transaction cache tier.</td></tr>
     * </table>
     *
     * @param key
     *            Object representing the key to be used for cache entry invalidation
     * @param isGlobal
     *            Indicates whether to remove the entry from the BackingMap (true) or just the ObjectMap (false).
     *
     * @throws IllegalArgumentException
     *             if key is <code>null</code>
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     */
    void invalidate(Object key, boolean isGlobal) throws ObjectGridException;

    /**
     * Invalidate a set of cache entries based on the Collection of keys provided. If a key in the collection cannot be
     * found, it will be ignored.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#INVALIDATE}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit except the Loader.
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.  
     * Set the <code>isGlobal</code> parameter to <code>false</code> to limit the operation to the transaction cache tier.</td></tr>
     * </table>
     *
     * @param keyList
     *            A Collection of keys representing the entries to be invalidated
     * @param isGlobal
     *            Indicates whether to remove the entry from the BackingMap (true) or just the ObjectMap (false).
     *
     * @throws IllegalArgumentException
     *             if keyList is null or contains a <code>null</code> element.
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #invalidate(Object, boolean)
     */
    void invalidateAll(Collection /* Object */keyList, boolean isGlobal) throws ObjectGridException;

    /**
     * Establishes the number of seconds that any given cache entry can live for, which is referred to as "time to live"
     * or TTL. Setting a new TTL value affects cache entries that are accessed after this method call occurs. It does
     * not affect any cache entry that was created or accessed prior to this method call. By calling this method on this
     * <code>ObjectMap</code>, any previous value set by the <code>BackingMap.setTimeToLive(int)</code> method is
     * overridden for this ObjectMap. If this method is never called on the ObjectMap, the default setting is used. The
     * default setting is to retain the time-to-live value for any existing map entry and to use the default value from
     * BackingMap setting if a new map entry is being created. If TTL is never set on the BackingMap, the cache entry
     * can live "forever".
     * <p>
     * This method can only be used when the <code>TTLType</code> is set to <code>LAST_ACCESS_TIME</code> or
     * <code>LAST_UPDATE_TIME</code> on the BackingMap.  If this method is called on the ObjectMap and the <code>TTLType</code>
     * is something other than <code>LAST_ACCESS_TIME</code> or <code>LAST_UPDATE_TIME</code>, an IllegalStateException is thrown.<p>
     *
     * Required permission: {@link MapPermission#INVALIDATE}
     *
     *
     * @param  ttl   is the time-to-live value in seconds.  The value must be >= -1. A value of 0
     *               is used to indicate the cache entry can live "forever" and -1 to indicate to
     *               use default setting. Use of the constant <code>TTL_FOREVER</code> is recommended
     *               when "forever" is desired and the constant<code>USE_DEFAULT</code> is recommended
     *               when "use default" setting is desired.
     *
     * @return previous time-to-live value in seconds. The constant <code>TTL_FOREVER</code> and
     *                  constant <code>USE_DEFAULT</code>can be used to determine if the previous
     *                  TTL is one of the special values.
     *
     * @throws IllegalArgumentException if seconds argument is < -1.
     * @throws IllegalStateException if <code>BackingMap.getTtlEvictorType()</code> returns anything other
     *         than <code>TTLType.LAST_ACCESS_TIME</code> or <code>TTLType.LAST_UPDATE_TIME</code>.
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #TTL_FOREVER
     * @see #USE_DEFAULT
     * @see BackingMap#setTimeToLive(int)
     * @see TTLType#LAST_ACCESS_TIME
     * @see TTLType#LAST_UPDATE_TIME
     */
    int setTimeToLive(int ttl);

    /**
     * A constant indicating the time-to-live value is "forever".
     */
    public static final int TTL_FOREVER = 0;

    /**
     * A constant indicating the time-to-live value or lock timeout value is the default setting.
     * <p>
     * The default setting is to retain the time-to-live value for any existing map entry and to use the
     * default value from BackingMap setting if a new map entry is being created.
     * <p>
     * For lock timeout override the default setting is to use the value defined on the BackingMap
     *
     * @see #setLockTimeout(int)
     * @see #setTimeToLive(int)
     * @see BackingMap#setTimeToLive(int)
     * @see BackingMap#getTimeToLive()
     * @see BackingMap#setLockTimeout(int)
     */
    public static final int USE_DEFAULT = -1;

    /**
     * Used as a parameter on the {@link #getNextKey(long)} method, specifies to return a null value if the map is
     * empty.
     */
    public static final long QUEUE_TIMEOUT_NONE = 0;

    /**
     * Used as a parameter on the {@link #getNextKey(long)} method, specifies the method should block until a key
     * becomes available.
     */
    public static final long QUEUE_TIMEOUT_INFINITE = Long.MAX_VALUE;

    /**
     * Performs an explicit update of a given entry.
     * <p>
     * A <code>get</code> operation is not required prior to invoking the <code>update</code> method (unlike the
     * <code>put</code> method). Also, an <code>update</code> invocation will never insert a new record. If a the
     * map's <code>LockStrategy</code> is <code>LockStrategy.OPTIMISTIC</code> this method will implicitly get the
     * entry so as to have the version value of the object for when this method was invoked. Whether or not a copy of
     * the object is made when transaction is committed is determined by the CopyMode setting for this map. See
     * <code>CopyMode</code> for a description of each possible CopyMode.
     * </p>
     *
     * <p>
     * If a key cannot be found in the map during commit, a TransactionException will be thrown.
     * </p><p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#WRITE}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit. 
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.</td></tr>
     * </table>
     *
     * @param key
     *            Identifies the entry to be updated
     * @param value
     *            The updated value for this entry
     * @throws IllegalArgumentException
     *             if key is <code>null</code> or if the map does not allow <code>null</code> values and value is
     *             <code>null</code>.
     * @throws KeyNotFoundException
     *             if the key cannot be found in the map
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #insert(Object, Object)
     * @see #put(Object, Object)
     * @see CopyMode
     * @see LockStrategy#OPTIMISTIC
     */
    void update(Object key, Object value) throws KeyNotFoundException, ObjectGridException;

    /**
     * Performs an explicit insert of a given entry.
     * <p>
     * The key must not exist before executing this method. Also, an <code>insert</code> invocation will never update
     * an existing record. Whether or not a copy of the object is made when a transaction is committed is determined by
     * the CopyMode setting for this map. See <code>CopyMode</code> for a description of each possible CopyMode.
     * </p>
     *
     * <p>
     * If the key is already in the map, a TransactionException will be thrown during commit.
     * </p><p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#INSERT}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit. 
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.</td></tr>
     * </table>
     *
     * @param key
     *            Identifies the entry to be inserted
     * @param value
     *            The value for this entry
     *
     * @throws IllegalArgumentException
     *             if key is <code>null</code> or if the map does not allow <code>null</code> values and value is
     *             <code>null</code>.
     * @throws DuplicateKeyException
     *             if this entries already exists in the map
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #put(Object, Object)
     * @see #update(Object, Object)
     * @see CopyMode
     */
    void insert(Object key, Object value) throws DuplicateKeyException, ObjectGridException;

    /**
     * Returns a reference to the named index that can be used with this Map. This index cannot be shared between
     * threads and works on the same rules as <code>Session</code>. The returned value should be cast to the right
     * index interface such as <code>MapIndex</code>, <code>MapRangeIndex</code> or a custom index interface such
     * as a geo spatial index.
     *
     * @param name
     *            The index name
     *
     * @return A reference to the index, it must be cast to the appropriate index interface.
     *
     * @throws IndexUndefinedException
     *             if the index is not defined on the BackingMap
     * @throws IndexNotReadyException
     *             if the index is a dynamic index and it is not ready
     *
     * @since WAS XD 6.0.1
     */
    Object getIndex(String name) throws IndexUndefinedException, IndexNotReadyException;

    /**
     * Returns a reference to the named index that can be used with this Map. This index cannot be shared between
     * threads and works on the same rules as <code>Session</code>. The returned value should be cast to the right
     * index interface such as <code>MapIndex</code>, <code>MapRangeIndex</code> or a custom index interface such
     * as a geo spatial index.
     *
     * @param name
     *            The index name
     * @param forUpdate
     *            if true, the returned index will always operate with forUpdate intent.
     *
     * @return A reference to the index, it must be cast to the appropriate index interface.
     *
     * @throws IndexUndefinedException
     *             if the index is not defined on the BackingMap
     * @throws IndexNotReadyException
     *             if the index is a dynamic index and it is not ready
     *
     * @since WAS XD 6.1.0.1
     */
    Object getIndex(String name, boolean forUpdate) throws IndexUndefinedException, IndexNotReadyException;

    /**
     * Pushes the current set of changes for the <code>ObjectMap</code> instance to the <code>Loader</code> without
     * committing the changes. The changes are not propagated to the <code>BackingMap</code> either. This is useful
     * for re-priming the <code>Loader</code>'s data without committing the current transaction and starting over.
     *
     * @throws ObjectGridException
     *             if an error occurs during processing
     *
     * @see Session#flush()
     */
    void flush() throws ObjectGridException;

    /**
     * Returns <code>true</code> if this map contains a mapping for the specified key. ObjectGrid does not support
     * null keys. If you configured the map to support <code>null</code> values, this method can be used to determine
     * whether a key is contained in the map or not.<p>
     *
     * This API does not hold any locks when using pessimistic locking.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#READ}</td></tr>
     * <tr><td valign="top">Pessimistic read lock acquired:</td><td valign="top">{@link LockMode#SHARED}</td></tr>
     * <tr><td valign="top">Pessimistic locks held:</td><td valign="top">None</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Progresses to all tiers until the key is found.</td></tr>
     * </table>
     *
     * @param key
     *            key whose presence in this map is to be tested.
     * @return <code>true</code> if this map contains a mapping for the specified key.
     *
     * @throws IllegalArgumentException
     *             if <code>null</code> key parameter is passed in
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     */
    boolean containsKey(Object key) throws ObjectGridException;

    /**
     * Returns an implementation of <code>java.util.Map</code> that is backed by this <code>ObjectMap</code>.
     * <p>
     * The returned <code>java.util.Map</code> implementation can be cast to
     * <code>com.ibm.websphere.objectgrid.JavaMap</code> to be able to use the rest of the ObjectGrid programming
     * model, but with <code>java.util.Map</code>'s use of <code>RuntimeException</code>s instead of checked
     * <code>ObjectGridException</code>s.
     *
     * @return a <code>java.util.Map</code> backed by this <code>ObjectMap</code>
     *
     * @see Map
     * @see JavaMap
     */
    Map getJavaMap();

    /**
     * Updates the last access time in the BackingMap without retrieving the value to the ObjectMap.
     * <p>
     * The last access time is updated during commit. If the key does not exist in the BackingMap, a
     * TransactionException will be returned during commit processing.
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">None</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit. 
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.</td></tr>
     * </table>
     *
     * @param key
     *            key to be touched
     *
     * @throws IllegalArgumentException
     *             if key is <code>null</code>
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     */
    void touch(Object key) throws ObjectGridException;

    /**
     * Allows the CopyMode for the Map to be overridden on this map on this session only.
     * <p>
     * This method allows an application to use an optimal CopyMode TRANSACTION by TRANSACTION as its needs dictate. The
     * CopyMode cannot be changed during a transaction. There must be no active transaction when this method is called.
     *
     * @param copyMode
     *            must be one of the final static variables defined in <code>CopyMode</code>. See
     *            <code>CopyMode</code> class for an explanation of each mode and how the valueInterface is used for
     *            <code>CopyMode.COPY_ON_WRITE</code> .
     * @param valueInterface
     *            the value interface Class object.  Specify null in version 7.1 and later.
     *
     * @throws IllegalArgumentException
     *             if copyMode is <code>null</code> or COPY_ON_WRITE CopyMode is specified and the required value
     *             interface parameter is <code>null</code>
     * @throws TransactionAlreadyActiveException
     *             if a transaction is active on the associated session
     * @throws ObjectGridException
     *             if an error occurs during processing
     *
     * @see BackingMap#setCopyMode(CopyMode, Class)
     * @see CopyMode
     */
    public void setCopyMode(CopyMode copyMode, Class valueInterface) throws TransactionAlreadyActiveException,
            ObjectGridException;

    /**
     * Resets the CopyMode back to the one in the BackingMap.
     * <p>
     * This method is used to reverse a previous setCopyMode method call for this ObjectMap. This method can only be
     * called when no transaction is active on the associated session.
     *
     * @throws TransactionAlreadyActiveException
     *             if a transaction is active on the associated session
     *
     * @see #setCopyMode(CopyMode, Class)
     */
    public void clearCopyMode() throws TransactionAlreadyActiveException;

    /**
     * Retrieves a key off the map in first-in-first-out (FIFO) insert order.
     * <p>
     * The entry is locked by the session such that other calls to getNextKey will not return the same key. The key can
     * be used to remove or manipulate the value although leaving the entry will result in the key remaining at the
     * beginning of the queue. This order is optimized for performance and is not guaranteed especially across
     * partitions or in highly concurrent environments.<p>
     * 
     * The return value is a {@link SerializedKey} when {@linkplain OutputFormat#RAW} is set for the keys.
     * The default key output format for maps that are associated with a {@linkplain KeySerializerPlugin} is {@linkplain OutputFormat#RAW}.
     * The SerializedKey allows access to the value in its serialized form, or its native Java Object form.<p>
     * 
     * The return value is a {@link Tuple} when an an EntityManager API entity is associated with the BackingMap.<p>
     *
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#READ}</td></tr>
     * <tr><td valign="top">Pessimistic read lock acquired:</td><td valign="top">{@link LockMode#EXCLUSIVE}</td></tr>
     * <tr><td valign="top">Pessimistic read lock held:</td><td valign="top">Yes</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * </table>
     *
     * @param timeout
     *            The period of time in milliseconds to wait for an entry to become available on the queue.
     *
     * @return the next key
     *
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @see #QUEUE_TIMEOUT_INFINITE
     * @see #QUEUE_TIMEOUT_NONE
     * @since WAS XD 6.1
     */
    public Object getNextKey(long timeout) throws ObjectGridException;

    /**
     * Retrieve the metadata for the entity associated with this map.
     * @return the EntityMetadata if an entity is associated with this map or null if there is no
     * entity associated with this map.
     * @since WAS XD 6.1
     */
    public EntityMetadata getEntityMetadata();

    /**
     * Returns the type of the underlying BackingMap.
     * <p>
     * The return value is equivalent to one of the constants declared on the BackingMap interface,
     * {@link BackingMap#LOCAL}, {@link BackingMap#SERVER}, or {@link BackingMap#CLIENT}.
     *
     * @return the BackingMap type
     *
     * @since WAS XD 6.1
     */
    public int getMapType();

    /**
     * Returns the Agent Manager that allows DataGrid operations to be performed on the objects within this Map.
     * <p>
     * This method should only be called on a client ObjectGrid. If called on a non client ObjectGrid an
     * <code>IllegalStateException</code> will be thrown
     *
     * @return AgentManager
     *
     * @throws IllegalStateException
     *             if this method is invoked on a non client ObjectGrid
     * @since WAS XD 6.1
     */
    public AgentManager getAgentManager();

    /**
     * Overrides the BackingMap's lock timeout for this ObjectMap.
     * <p>
     * Establishes the number of seconds that any given fetch (get, getForUpdate, find, findForUpdate) of a cache entry
     * will wait to get a lock. When the lock strategy is <code>LockStrategy.NONE</code>, no lock manager is used by
     * this map. In this case, a call to this method does nothing.
     *
     * @param seconds
     *            is the lock timeout in seconds. The value must be >= -1. A value of -1 is used to indicate to use
     *            the default setting. Use of the constant<code>USE_DEFAULT</code> is recommended when "use default"
     *            setting is desired.  A value of 0 indicates that if a lock cannot be retrieved immediately to time out
     *            without waiting for any period of time for the lock to be released and made available.
     *
     * @throws IllegalArgumentException
     *             if seconds argument is less than -1 (USE_DEFAULT)
     *
     * @see #USE_DEFAULT
     * @see BackingMap#setLockTimeout(int)
     * @see BackingMap#setLockStrategy(LockStrategy)
     * @see LockStrategy#OPTIMISTIC
     * @see LockStrategy#PESSIMISTIC
     *
     * @since WAS XD 6.1
     */
    public void setLockTimeout(int seconds);

    /**
     * Clear all keys from the Map.
     * <p>
     * This method is an automatic transaction call.  The {@link Session#isTransactionActive()} must answer false prior
     * to invoking this method.<p>
     * 
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#REMOVE}</td></tr>
     * <tr><td valign="top">Pessimistic read lock acquired:</td><td valign="top">Map exclusive lock is acquired.</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Hybrid.  Each shard is cleared in a separate transaction.</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applies to all cache tiers except the Loader.</td></tr>
     * </table>
     *
     * @throws ObjectGridException if an error occurs during processing
     * @throws TransactionAlreadyActiveException if a transaction is already started.
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     *
     * @since WAS XD 6.1.0.3
     */
    public void clear() throws ObjectGridException;

    /**
     * Obtains a lock for the given key.
     * 
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#READ}</td></tr>
     * <tr><td valign="top">Pessimistic read lock acquired:</td><td valign="top">{@link LockMode#SHARED}, {@link LockMode#UPGRADABLE} or {@link LockMode#EXCLUSIVE}<br>Note:  Transaction isolation is ignored.</td></tr>
     * <tr><td valign="top">Pessimistic locks held:</td><td valign="top">Yes</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Progresses to all tiers until the key is found and the appropriate lock is acquired.</td></tr>
     * </table>
     *
     * @param key
     *            the key to lock
     * @param lockMode
     *            the lockMode to obtain for the given key
     * @return true if the entry exists in the grid or Loader (if one is defined)
     * @throws IllegalArgumentException
     *             if key is <code>null</code>
     * @throws IllegalStateException if this map is not using {@link LockStrategy#PESSIMISTIC} LockStategy
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     * @since 8.6
     */
    public boolean lock(Object key, LockMode lockMode) throws ObjectGridException;

    /**
     * Obtains locks for the given keys.
     * 
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#READ}</td></tr>
     * <tr><td valign="top">Pessimistic read lock acquired:</td><td valign="top">{@link LockMode#SHARED}, {@link LockMode#UPGRADABLE} or {@link LockMode#EXCLUSIVE}<br>Note:  Transaction isolation is ignored.</td></tr>
     * <tr><td valign="top">Pessimistic locks held:</td><td valign="top">Yes</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Progresses to all tiers until the keys are found and the appropriate locks are acquired.</td></tr>
     * </table>
     *
     * @param keyList
     *            the keys to lock
     * @param lockMode
     *            the lockMode to obtain for the given keys
     * @return a List of <code>Boolean</code>s indicating whether the entries exists in the grid or Loader (if one is
     *         defined)
     * @throws IllegalStateException if this map is not using {@link LockStrategy#PESSIMISTIC} LockStategy
     * @throws IllegalArgumentException
     *             if keyList is null or contains a <code>null</code> element.
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     * @since 8.6
     */
    public List<Boolean> lockAll(List keyList, LockMode lockMode) throws ObjectGridException;

    /**
     * Puts the Object value into the cache at location represented by key.<p>
     * 
     * The value will be pushed down to the BackingMap/Loader at commit time. The semantics of this method are that 
     * the Loader will receive a {@link LogElement#UPSERT} operation and the map will either do an insert or an update
     * to cause the map to contain this updated value.
     *
     * Whether or not a copy of the object is made when transaction is committed is determined by the copy mode setting
     * for this map. See <code>CopyMode</code> for a description of each possible copy mode.<p>
     * 
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#WRITE}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit. 
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.</td></tr>
     * </table>
     *
     * @param key
     *            The entry to insert or update in the map
     * @param value
     *            The value to insert or update in the map using the key
     *
     * @throws IllegalArgumentException
     *             if key is <code>null</code>, or if the map does not allow <code>null</code> values and value is
     *             <code>null</code>
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     * @see CopyMode
     * @since 8.6
     */
    public void upsert(Object key, Object value) throws ObjectGridException;

    /**
     * Puts each of the Object value into the cache at location represented by key contained in the Map.
     * 
     * The values will be pushed down to the BackingMap/Loader at commit time. The semantics of this method are that 
     * the Loader will receive a {@link LogElement#UPSERT} operation and the map will either do an insert or an update
     * to cause the map to contain this updated value.
     *
     * Whether or not a copy of the objects is made when transaction is committed is determined by the copy mode setting
     * for this map. See <code>CopyMode</code> for a description of each possible copy mode.<p>
     * 
     * <strong>Specification details:</strong>
     * <table>
     * <tr><td valign="top">Required permission:</td><td valign="top">{@link MapPermission#WRITE}</td></tr>
     * <tr><td valign="top">Transaction:</td><td valign="top">Automatic or manual</td></tr>
     * <tr><td valign="top">Cache tier:</td><td valign="top">Applied to all tiers during commit. 
     * Use {@link Session#beginNoWriteThrough} to limit the operation to the Client Cache tier for client maps, or the
     * Server Cache tier for local or shard maps.</td></tr>
     * </table>
     *
     * @param map
     *            The key/values to be inserted or updated in the map.  The type is LinkedHashMap so that
     *            the order can be controlled to avoid deadlocks.
     *
     * @throws IllegalArgumentException
     *             if map is <code>null</code> or contains a <code>null</code> key or if <code>null</code> values
     *             are not allowed and map contains a <code>null</code> value.
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @throws AccessControlException
     *             if the Subject or Credential specified on the Session does not have the appropriate permission.
     * @see CopyMode
     * @since 8.6
     */
    public void upsertAll(LinkedHashMap map) throws ObjectGridException;
    
    
    /**
     * Retrieves the data format for all data access APIs that return cache keys for the current session.
     * 
     * @return the data output format or null if the default should be used.
     * @since 8.6
     */
    public OutputFormat getKeyOutputFormat();
    
    /**
     * Sets the data format for all data access APIs that return cache keys for the current session.<p>
     * 
     * This method supports map configurations with a {@link KeyDataSerializer} plug-in defined, or with
     * eXtreme Data Format enabled.<p>
     *  
     * @param outputFormat the data output format to use or {@link OutputFormat#UNDEFINED}  to use the default defined 
     * on the parent {@link BackingMap}.
     * 
     * @throws IllegalArgumentException thrown when the data format is not valid for the current configuration.
     * @throws TransactionAlreadyActiveException
     *             if a transaction is active on the associated session
     * @see BackingMap#getKeyOutputFormat()
     * @since 8.6
     */
    public void setKeyOutputFormat(OutputFormat outputFormat) throws TransactionAlreadyActiveException;

    /**
     * Retrieves the data format for all data access APIs that return cache values for the current session.<p>
     * 
     * @return the data output format or null if the default should be used.
     * @since 8.6
     */
    public OutputFormat getValueOutputFormat();

    /**
     * Sets the data format for all data access APIs that return cache values for the current session.<p>
     * 
     * This method is functionally equivalent to the {@link #setCopyMode(CopyMode, Class)} with the 
     * {@link CopyMode#COPY_TO_BYTES} and {@link CopyMode#COPY_TO_BYTES_RAW} values when used with a 
     * {@link ValueDataSerializer} or eXtreme Data Format.<p>
     * 
     * This method supports map configurations with a {@link ValueDataSerializer} plug-in defined, or with
     * eXtreme Data Format enabled.<p> 
     * 
     * @param outputFormat the data output format to use or {@link OutputFormat#UNDEFINED}  to use the default defined 
     * on the parent {@link BackingMap}.
     * 
     * @throws IllegalArgumentException thrown when the data format is not valid for the current configuration.
     * @throws TransactionAlreadyActiveException
     *             if a transaction is active on the associated session
     * @since 8.6
     */
    public void setValueOutputFormat(OutputFormat outputFormat) throws TransactionAlreadyActiveException;  
    
    /**
     * Identifies the operation mode of the {@link ObjectMap#put(Object, Object)}, {@link ObjectMap#putAll(Map)}
     * , {@link JavaMap#put(Object, Object)} and {@link JavaMap#putAll(Map)} methods.
     * @since 8.6
     * @see ObjectMap#setPutMode(PutMode)
     */
    public enum PutMode {
        /**
         * The put API behaves like an insert or update.
         * @deprecated Deprecated in 8.6.  Use the {@link #UPSERT} enumeration.
         */
        INSERTUPDATE,
        
        /**
         * The put API behaves like the upsert method.
         */
        UPSERT
    }
    
    /**
     * Allows the default operation for the {@link #put(Object, Object)} and {@link #putAll(Map)} methods
     * to be changed, allowing the <code>put</code> to use the optimized 
     * {@link #upsert(Object, Object)} and {@link #upsertAll(LinkedHashMap)} implementations. <p>
     * 
     * The <c>PutMode</c> cannot be changed during a transaction. There must be no active transaction when this method is called.
     *
     * @param putMode
     *            the mode in which the <code>put</code> methods operate.
     *
     * @throws TransactionAlreadyActiveException
     *             if a transaction is active on the associated session
     * @throws ObjectGridException
     *             if an error occurs during processing
     * @since 8.6
     */
    public void setPutMode(PutMode putMode) throws TransactionAlreadyActiveException, ObjectGridException;
    
    /**
     * Retrieve the current <code>PutMode</code> set for this ObjectMap instance.
     * @return the current <code>PutMode</code>.
     * @since 8.6
     */
    public PutMode getPutMode();
    
}
