// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006-2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  QueryMapping.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b834         6.1       04/27/2006   jian.tang Single query configuration
// b3536        6.1       01/19/2007   cdjohnson Fix minor bug with access type detection
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.config;

import java.io.Serializable;

/**
 * A QueryMapping maps a Java class to a BackingMap and allows a map to be included in a 
 * query.  It also indicates whether the query engine should use a getter method or direct field access 
 * to access fields in the value class. <p>
 * 
 * For example, class Department is the value class that is stored in the "DepartmentMap" BackingMap and the
 * key is an Integer.
 * <pre>
 * public class Department {
 *     private int id;
 *     private Collection emps;
 *     
 *     public void setEmps(Collection emps) {
 *         this.emps = emps;
 *     }
 *     
 *     public Collection getEmps() {
 *         return emps;
 *     }
 *     ...
 * }
 * </pre>
 * 
 * The QueryMapping would be created as follows:
 * <pre>
 * ...
 * QueryConfig queryConfig = new QueryConfig();
 * queryConfig.addMapping(new QueryMapping(
 *     "DepartmentMap", Department.class.getName(), "id", QueryMapping.PROPERTY_ACCESS)
 * objectGrid.setQueryConfig(queryConfig);
 * ...
 * </pre>
 * 
 * @since WAS XD 6.1, XC10
 *
 */
public class QueryMapping implements Serializable {

    private static final long serialVersionUID = -2446303840405182304L;

    /**
     * This constant indicates to use direct field access to read the field values
     * @see #setAccessType(int) 
     * @see #getAccessType() 
     */
    public static final int FIELD_ACCESS = 0;
    
    /** 
     * This constant indicates to use JavaBean property-style get methods to read the field values
     * from the Java object stored in the BackingMap.  PROPERTY_ACCESS is the default. 
     * @see #setAccessType(int) 
     * @see #getAccessType() 
     */
    public static final int PROPERTY_ACCESS = 1;
    
    private String mapName;

    private String valueClass;
    
    private int accessType;
    
    private String primaryKeyField;

    /**
     * Default constructor.
     */
    public QueryMapping() {
        super();
    }

    /**
     * Constructor for creating a basic QueryMapping instance with a default
     * access type of {@link #PROPERTY_ACCESS}.<p>
     * 
     * The mapName and valueClass must not be null.
     * 
     * @param mapName the name of the BackingMap to map 
     * @param valueClass the class of object stored in the BackingMap's value.
     * @param primaryKeyField the optional name of the primary key field of the class. 
     */
    public QueryMapping(String mapName, String valueClass, String primaryKeyField) {
        super();
        this.mapName = mapName;
        this.valueClass = valueClass;
        this.primaryKeyField = primaryKeyField;
        accessType = PROPERTY_ACCESS;
    }
    
    /**
     * Constructor for creating a QueryMapping instance.
     * 
     * The mapName and valueClass must not be null.
     * @param mapName the name of the BackingMap to map 
     * @param valueClass the class of object stored in the BackingMap's value.
     * @param primaryKeyField the optional name of the primary key field of the class.
     * @param accessType the method ({@link #PROPERTY_ACCESS} or {@link #FIELD_ACCESS}) 
     * in which the query engine will access the persistent data in the value object.   
     */
    public QueryMapping(String mapName, String valueClass, String primaryKeyField, int accessType) {
        super();
        this.mapName = mapName;
        this.valueClass = valueClass;
        this.primaryKeyField = primaryKeyField;
        this. accessType = accessType;
        if (accessType != PROPERTY_ACCESS && accessType != FIELD_ACCESS) {
            throw new IllegalArgumentException("The access type has to be either QueryMapping.METHOD_ACCESS or QueryMapping.FIELD_ACCESS.");
        }
    }

    /**
     * Retrieve the BackingMap name associated with this mapping.
     * @return the BackingMap name.
     */
    public String getMapName() {
        return mapName;
    }

    /**
     * Set the BackingMap name associated with this mapping.
     */
    public void setMapName(String mapName) {
        this.mapName = mapName;
    }

    /**
     * Retrieve the type of object that is stored in the BackingMap.
     * @return the object type that is stored in the BackingMap's value.
     */
    public String getValueClass() {
        return valueClass;
    }

    /**
     * Set the type of object that is stored in the BackingMap.
     */
    public void setValueClass(String valueClass) {
        this.valueClass = valueClass;
    }

    /**
     * Retrieve the method in which the query engine will access the 
     * value class object stored in the BackingMap.
     * @return Returns the accessType.
     * @see #PROPERTY_ACCESS
     * @see #FIELD_ACCESS
     */
    public int getAccessType() {
        return accessType;
    }

    /**
     * Set the method in which the query engine will access the 
     * value class object stored in the BackingMap.
     * @param accessType the accessType.
     * @see #PROPERTY_ACCESS
     * @see #FIELD_ACCESS
     */
    public void setAccessType(int accessType) {
        this.accessType = accessType;
    }

    
    /**
     * Retrieve the name of the attribute in the value class object that 
     * is also the primary key of the BackingMap.<p>
     * This value is optional.
     * @return the primaryKeyField.
     */
    public String getPrimaryKeyField() {
        return primaryKeyField;
    }

    /**
     * Set the name of the attribute in the value class object that 
     * is also the primary key of the BackingMap.
     * @param primaryKeyField the name of the primary key attribute or null if not set.
     */
    public void setPrimaryKeyField(String primaryKeyField) {
        this.primaryKeyField = primaryKeyField;
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }

        if (o instanceof QueryMapping) {
            QueryMapping other = (QueryMapping) o;

            return mapName.equals(other.mapName)
                    && valueClass.equals(other.valueClass)
                    && (primaryKeyField == null ? other.primaryKeyField == null : primaryKeyField
                            .equals(other.primaryKeyField)) && accessType == other.accessType;
        }

        return false;

    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return mapName.hashCode() + valueClass.hashCode() + (primaryKeyField == null ? 0 : primaryKeyField.hashCode())
                + accessType;
    }

    public String toString() {
        return super.toString() + "[" + mapName + ", " + valueClass + ", " + primaryKeyField + ", " + accessType + "]";
    }

}
