/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * b8216         6.1      05/30/2007   josh      new part
 * b10338        6.1      10/27/2007   haowang   catalog server control
 * b13177        6.1.0.5  11/07/2008   vandersn  Add tiggerPlacement and replaceLostShards
 * b21165        7.1      09/17/2010   mdgann    java doc fixes
 */

package com.ibm.websphere.objectgrid.management;

import java.util.List;

import javax.management.openmbean.TabularData;

/**
 * This MBean interface allows a client process to perform operations on and get status from the PlacementService
 * running in a dynamic environment. The object name pattern for this MBean is:
 *
 * <pre>
 * com.ibm.websphere.objectgrid:type=PlacementService
 * </pre>
 *
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object
 * name.
 *
 * @since WAS XD 6.1 FIX3, XC10
 */

public interface PlacementServiceMBean extends CoreGroupServiceMBean {

    /**
     * Constant representing a primary shard type.
     */
    public static final int PRIMARY = 1;

    /**
     * Constant representing a synchronous replica shard type.
     */
    public static final int SYNCHRONOUS_REPLICA = 2;

    /**
     * Constant representing an asynchronous replica shard type.
     */
    public static final int ASYNCHRONOUS_REPLICA = 4;

    /**
     * Constant representing a all shard types
     */
    public static final int ALL = 0xFF;


    /**
     * Retrieves a List of JMX address strings for a specific host and server name that has registered with the
     * placement service.
     *
     * @param hostName
     *            The name of the host to retrieve the JMX addresses.
     * @param serverName
     *            The name of the server to retrieve the JMX addresses.
     * @return the List of all JMX address strings for the specified host and server name.
     */
    public List retrieveServerJMXAddress(String hostName, String serverName);

    /**
     * Retrieves TabularData of JMX address strings for a specific host and server name that has registered with the
     * placement service. Null host or null server names can be used to retrieve several results. Using a null host and
     * null server name will retrieve all of the servers.
     * 
     * The TabularData contains CompositeData with the following items, where each CompositeData represents a server:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr> <td>JMXServiceURL</td><td>String</td><td>JMX Service URL</td> </tr>
     * <tr> <td>HostName</td><td>String</td><td>Host name</td> </tr>
     * <tr> <td>ServerName</td><td>String</td><td>Server Name</td> </tr>
     * </table>
     *
     * @param hostName
     *            The name of the host for which to retrieve JMX addresses. Use null or an empty string to retrieve JMX
     *            addresses for any host.
     * @param serverName
     *            The name of the server for which to retrieve JMX addresses. Use null or an empty string to retrieve JMX
     *            addresses for any server.
     * @return the TabularData of all JMX address strings for the specified host and server name.
     * @since 8.5
     */
    public TabularData retrieveServerJMXAddressesWithInfo(String hostName, String serverName);

    /**
     * Retrieves a List of JMX addresses for all servers that have registered with the placement service.
     *
     * @return the List of all servers' JMX addresses
     */
    public List retrieveAllServersJMXAddresses();

    /**
     * Retrieves the container status for all containers in the domain.
     * <p>
     *
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;container name="&lt;container&gt;" zoneName="&lt;zone&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"&gt;
     *   &lt;shard type="&lt;type&gt;" partitionName="&lt;partition&gt;"/&gt;
     * &lt;/container&gt;
     * </pre>
     *
     * @param objectGridName
     *            The name of the ObjectGrid for which to get container status.
     * @param mapSetName
     *            The name of the mapset for which to get the container status.
     * @return The String status object for all containers in XML form.
     */
    public String collectContainerStatus(String objectGridName, String mapSetName);

    /**
     * List the placement of shards for each container in the domain.
     * <p>
     *
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset"&gt;
     *   &lt;container name="&lt;container&gt;" zoneName="&lt;zone&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"&gt;
     *      &lt;shard type="&lt;type&gt;" partitionName="&lt;partition&gt;" reserved="&lt;true&gt;"/&gt;
     *   /container&gt;
     * &lt;/objectGrid&gt;
     *  </pre>
     *<b> NOTE:</b> The default value for the "reserved" attribute is false.
     *
     * @param objectGridName
     *            The name of the ObjectGrid for which to get placement status.
     * @param mapSetName
     *            The name of the mapset for which to get the placement status.
     * @return The placement status in XML form.
     */
    public String listObjectGridPlacement(String objectGridName, String mapSetName);

    /**
     * List the current placement status.
     * <p>
     *
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;configuration&gt;
     *     &lt;attribute name="&lt;placementStrategy&gt;" value="&lt;strategy&gt;"/&gt;
     *     &lt;attribute name="&lt;numInitialContainers&gt;" value="&lt;num&gt;"/&gt;
     *     &lt;attribute name="&lt;minSyncReplicas&gt;" value="&lt;min&gt;"/&gt;
     *     &lt;attribute name="&lt;developmentMode&gt;" value="&lt;mode&gt;"/&gt;
     *   &lt;/configuration&gt;
     *   &lt;runtime&gt;
     *     &lt;attribute name="&lt;numContainers&gt;" value="&lt;num&gt;"/&gt;
     *     &lt;attribute name="&lt;numMachines&gt;" value="&lt;num&gt;"/&gt;
     *     &lt;attribute name="&lt;numOutstandingWorkItems&gt;" value="&lt;num&gt;"/&gt;
     *     &lt;attribute name="&lt;numActiveZones&gt;" value="&lt;num&gt;"/&gt;
     *   &lt;/runtime&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     *
     * @param objectGridName
     *            The name of the ObjectGrid for which to get placement status.
     * @param mapSetName
     *            The name of the mapset for which to get the placement status.
     * @return The placement status in XML form.
     */
    public String listObjectGridPlacementStatus(String objectGridName, String mapSetName);

    /**
     * Gets the coregroup status.
     * <p>
     *
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;coreGroup name="&lt;coregroup&gt;"&gt;
     *   &lt;coreGroupLeader hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     *   &lt;coreGroupMember hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     * &lt;/coreGroup&gt;
     * </pre>
     *
     * @return the coregroup status in XML form.
     */
    public String getCoreGroups();

    /**
     * List the coregroup members for a given coregroup.
     * <p>
     *
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;coreGroup name="&lt;coregroup&gt;"&gt;
     *   &lt;coreGroupMember hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     * &lt;/coreGroup&gt;
     * </pre>
     *
     * @param coreGroupName
     *            The name of the coregroup for which to get the members.
     * @return The coregroup members in XML form.
     */
    public String listCoreGroupMembers(String coreGroupName);

    /**
     * List the partition placement status in the domain.
     *
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;partition name="&lt;partition&gt;"&gt;
     *   &lt;shard type="&lt;type&gt;" containerName="&lt;container&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     * &lt;/partition&gt;
     * </pre>
     *
     * @param objectGridName
     *            The name of the ObjectGrid for which to get placement status.
     * @param mapSetName
     *            The name of the mapset for which to get the placement status.
     * @param partitionId
     *            The name of the partition for which to get the placement status.
     * @return The partition placement status in the XML form.
     */
    public String listPartition(String objectGridName, String mapSetName, String partitionId);

    /**
     * List the shard placement status.
     * <p>
     *
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;container name="&lt;container&gt;" zoneName="&lt;zone&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"&gt;
     *     &lt;shard type="&lt;type&gt;" partitionName="&lt;partition&gt;"/&gt;
     *   &lt;/container&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     *
     * @param objectGridName
     *            The name of the ObjectGrid for which to get placement status.
     * @param mapSetName
     *            The name of the mapset for which to get the placement status.
     * @param containerName
     *            The name of the container for which to get the placement status. If empty string ( "" ), get shard
     *            placement for all containers.
     * @param mask
     *            The Integer mask to determine for which shard types to get status.
     * @see #ALL
     * @see #PRIMARY
     * @see #SYNCHRONOUS_REPLICA
     * @see #ASYNCHRONOUS_REPLICA
     * @return The shard placement status in XML form.
     */
    public String listShards(String objectGridName, String mapSetName, String containerName, int mask);

    /**
     * Gets the names of all ObjectGrids and their mapsets in the domain.
     * <p>
     *
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"/&gt;
     * </pre>
     *
     * @return the names of all ObjectGrids and their mapsets in the domain in XML form.
     */
    public String getObjectGridNames();

    /**
     * Lost shards are placed onto the UNREPAIRED container when autoReplaceLostShards is disabled. Shards on the
     * UNREPAIRED will not be placed until this method is called.
     * <p>
     *
     * Calling this method will move shards off the UNREPAIRED container onto the UNASSIGNED container.
     * <p>
     *
     * Balance and placement operations will be queued up for the MapSet specified. These operations will execute when
     * all outstanding placement work from previous events has completed.
     * <p>
     *
     * The string returned is an XML representation of the shards that moved as a result of the call to this method.
     * <p>
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;shard type="&lt;type&gt;" partitionName="&lt;partition&gt;"&gt;
     *     &lt;currentContainer name="&lt;container&gt;" zoneName="&lt;zone&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     *     &lt;previousContainer name="&lt;container&gt;" zoneName="&lt;zone&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     *   &lt;/shard&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     *
     * <p>
     * The returned XML will look as follows when no shards have been moved:
     * <p>
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;!-- No shards were moved --&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     *
     * @param objectGridName
     *            replace lost shards for this ObjectGrid
     * @param mapSetName
     *            replace lost shards for this MapSet
     * @return An XML String containing shards that have moved
     * @since WAS XD 6.1.0.5
     */
    public String replaceLostShards(String objectGridName, String mapSetName);

    /**
     * Placement normally occurs implicitly after an event such as an ObjectGrid container starting or stopping.
     * <p>
     *
     * Calling this method will trigger a placement operation for the ObjectGrid and MapSet specified.
     * <p>
     *
     * Under normal circumstances, the <tt>numInitialContainers</tt> attribute (in the deployment policy) must be met
     * in order for placement to occur. However, when this method is called, the numInitialContainers value is ignored.
     * <p>
     *
     * The string returned is an XML representation of the shards that moved as a result of the call to this method.
     * <p>
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;shard type="&lt;type&gt;" partitionName="&lt;partition&gt;"&gt;
     *     &lt;currentContainer name="&lt;container&gt;" zoneName="&lt;zone&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     *     &lt;previousContainer name="&lt;container&gt;" zoneName="&lt;zone&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     *   &lt;/shard&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     *
     * <p>
     * The returned XML will look as follows when no shards have been moved:
     * <p>
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;!-- No shards were moved --&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     *
     * @param objectGridName
     *            trigger placement for this ObjectGrid
     * @param mapSetName
     *            trigger placement for this MapSet
     * @return An XML String containing shards that have moved
     * @see com.ibm.websphere.objectgrid.deployment.ObjectGridDeployment#addMapSet(com.ibm.websphere.objectgrid.deployment.MapSet)
     * @see com.ibm.websphere.objectgrid.deployment.MapSet#setNumInitialContainers(int)
     * @since WAS XD 6.1.0.5
     */
    public String triggerPlacement(String objectGridName, String mapSetName);


    /**
     * Each of the container servers that are passed into this method will be stopped.  If the server cannot be reached,
     * all of the server's artifacts will be removed.<p>
     *
     * Use this method if servers are found to be in a corrupt state or bindings need to be cleared
     * from the catalog server.<p>
     *
     * The string returned is an XML representation of the results of the attempt to tear down each of the servers.  If
     * the command is successful, the XML will look as follows:<p>
     *
     * <pre>
     * &lt;domain name="&lt;domain&gt;"&gt;
     *   &lt;server name="&lt;server&gt;" tearDownSuccessful="true"/&gt;
     *   &lt;server name="&lt;server&gt;" tearDownSuccessful="true"/&gt;
     * &lt;/domain&gt;
     * </pre>
     *
     * If the command is not successful, the string will look as follows (where the exception element is only
     * present if an exception is part of the failure):<p>
     *
     * <pre>
     * &lt;domain name="&lt;domain&gt;"&gt;
     *   &lt;server name="&lt;server&gt;" tearDownSuccessful="false" reason="&lt;String&gt;"&gt;
     *       &lt;exception type="&lt;String&gt;" message="&lt;String&gt;" stack="&lt;String&gt;"/&gt;
     *   &lt;/server&gt;
     * &lt;/domain&gt;
     * </pre>
     *
     * @param servers String array of servers to tear down.
     * @return An XML String containing the results of tear down attempts.
     * @since WAS XD 6.1.0.5 FIX2
     */
    public String tearDownServers(String[] servers);

    /**
     * <p>This method is deprecated. The com.ibm.websphere.objectgrid.client.RouteTableValidation utility replaces this method.</p>
     * 
     * Calling this method will return an XML string of the current known routing table. The Placement service will
     * contact each shard and return state on whether it was able to verify that's shard's existence. All shards will be
     * included in the XML doc, whether they were reachable or not. The user can use the <tt>reachable</tt> attribute
     * below to filter valid or invalid shards.
     *
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" name="&lt;name&gt;"&gt;
     *   &lt;primary zone="&lt;zone&gt;"&gt partition="&lt;partition&gt;"&gt state="&lt;reachable&gt;"&gt ipaddress="&lt;ipaddress&gt;"&gt;
     *     &lt;replica zone="&lt;zone&gt;"&gt partition="&lt;partition&gt;"&gt state="&lt;reachable&gt;"&gt ipaddress="&lt;ipaddress&gt;"&gt;
     *   &lt;/primary&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     *
     * @param objectGridName
     *            retrieve routing table for this ObjectGrid
     * @return An XML String containing a pre-verified routing table
     * @since WAS XD 6.1.0.5 FIX2
     */
    @Deprecated
    public String listVerifiedRoutingTable(String objectGridName);

    /**
     * Retrieves the name of the MapSet in which the specified map is defined.
     *
     * @param gridName the name of the ObjectGrid
     * @param mapName the name of the map
     * @return the name of the MapSet in which the specified map is defined.
     * @since 7.0
     */
    public String retrieveMapSetName(String gridName, String mapName);

    /**
     * Returns whether this instance of the Placement Service finished initialization.
     * 
     * @return true if initialized, false if not initialized.
     * @since 8.5
     */
    public boolean getInitialized();
    
    /**
     * The placement service will examine the distribution of primaries and replicas for a given mapSet 
     * and attempt (if zone rules and other balancing constraints allow) to achieve a consistent primary 
     * to replica ratio across the set of containers.
     * <p>
     * 
     * If the number of primaries or the number of replicas do not divide evenly across the containers, 
     * some tolerance must be allowed for the ratio to differ from container to container.  However, 
     * the difference in the number of primaries from one container to the next will not be greater 
     * than 1.  Similarly, the difference in the number of replicas from one container to the next will 
     * not be greater than 1.
     * <p>
     * 
     * Null arguments are not allowed as input to this method.
     * <p>
     * 
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;shard type="&lt;type&gt;" partitionName="&lt;partition&gt;"&gt;
     *     &lt;currentContainer name="&lt;container&gt;" zoneName="&lt;zone&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     *     &lt;previousContainer name="&lt;container&gt;" zoneName="&lt;zone&gt;" hostName="&lt;host&gt;" serverName="&lt;server&gt;"/&gt;
     *   &lt;/shard&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     * 
     * If no shards were moved or a problem was encountered attempting to execute this method, no shard elements will appear
     * in the XML output.  A detail element will appear instead.  The message attribute will have further information.
     * 
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;detail message="&lt;message&gt;" /&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     *
     * @param objectGridName
     *      the grid
     * @param mapSetName
     *      the map set within the grid
     * @return An XML String containing the results of the attempt to redistribute shards for better primary/replica balance
     *
     * @since 7.1.1
     */
    public String balanceShardTypes(String objectGridName, String mapSetName);

    /**
     * Prevent future balancing attempts for a specific map set.
     * Balancing work that is in progress will be allowed to complete.
     * <p>
     *
     * Other placement activities are allowed to execute while balancing is suspended.
     * <ul>
     * <li>shard promotion due to container loss</li>
     * <li>shard role swap</li>
     * <li>shard reservation</li>
     * <li>triggerPlacement</li>
     * <li>replaceLostShards</li>
     * </ul>
     * <p>
     *
     * Balancing will remain suspended until it is resumed by calling {@link #resumeBalancing(String, String)}.
     * <p>
     * 
     * Null arguments are not allowed as input to this method.
     * <p>
     * 
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;suspendBalancing currentValue="&lt;currentValue&gt;" previousValue="&lt;previousValue&gt;"/&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     * 
     * <p>
     * Additionally, an optional detail element may be contained within the suspendBalancing element.  The detail
     * element will include additional data regarding execution of this method.  The XML result will be in the following format
     * when a detail element is included:
     * 
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;suspendBalancing currentValue="&lt;currentValue&gt;" previousValue="&lt;previousValue&gt;"&gt;
     *     &lt;detail message="&lt;message&gt;" /&gt;
     *   &lt;suspendBalancing/&gt;
     * &lt;/objectGrid&gt;
     * </pre> 
     *
     * @param objectGridName
     *      suspend balancing for the map set specified within this ObjectGrid
     * @param mapSetName
     *      suspend balancing for this map set
     * @return An XML String containing the results of the attempt to suspend balancing
     *
     * @see #resumeBalancing(String, String)
     * @since 7.1.0.3
     */
    public String suspendBalancing(String objectGridName, String mapSetName);

    /**
     * Execute balancing operation at next opportunity and allow execution of future balancing attempts for
     * the map set specified.  Balancing is executed in reaction to key placement events.
     * Such events include containers starting and containers stopping.
     * <p>
     *
     * By default, balancing work is executed unless {@link #suspendBalancing(String, String)} has been called
     * for the map set.
     * <p>
     *
     * Null arguments are not allowed as input to this method.
     * <p>
     * 
     * The results are returned in the following format:
     *
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;suspendBalancing currentValue="&lt;currentValue&gt;" previousValue="&lt;previousValue&gt;"/&gt;
     * &lt;/objectGrid&gt;
     * </pre>
     * 
     * <p>
     * Additionally, an optional detail element may be contained within the suspendBalancing element.  The detail
     * element will include additional data regarding execution of this method.  The XML result will be in the following format
     * when a detail element is included:
     * 
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;suspendBalancing currentValue="&lt;currentValue&gt;" previousValue="&lt;previousValue&gt;"&gt;
     *     &lt;detail message="&lt;message&gt;" /&gt;
     *   &lt;suspendBalancing/&gt;
     * &lt;/objectGrid&gt;
     * </pre> 

     *
     * @param objectGridName
     *      resume balancing for the map set specified within this ObjectGrid
     * @param mapSetName
     *      resume balancing for this map set
     * @return An XML String containing the results of the attempt to resume balancing
     *
     * @see #suspendBalancing(String, String)
     * @since 7.1.0.3
     */
    public String resumeBalancing(String objectGridName, String mapSetName);

    /**
     * Check the balance status (suspended or resumed) for a specified MapSet.
     * <p>
     *
     * Null arguments are not allowed as input to this method.
     * <p>
     * 
     * The string returned is an XML representation of the balance status.  The XML will look as follows:<p>
     * 
     * <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;balanceStatus suspended="&lt;suspended&gt;" /&gt;
     * &lt;/objectGrid&gt;
     *  </pre>
     *  
     *  <p>
     *  Additionally, an optional detail element may be contained within the balanceStatus element.  When balancing has been pre-suspended, 
     *  the message attribute of the detail element will contain the following message. <p>
     *  
     *  <pre>
     * &lt;objectGrid name="&lt;objectgrid&gt;" mapSetName="&lt;mapset&gt;"&gt;
     *   &lt;balanceStatus suspended="true" &gt;
     *     &lt;detail message="Balancing has been pre-suspended for this mapSet." /&gt;
     *   &lt;/balanceStatus&gt;
     * &lt;/objectGrid&gt; 
     *  </pre>
     *  
     *  
     *
     * @param objectGridName
     *      check balance status for the map set specified within this ObjectGrid
     * @param mapSetName
     *      check balance status for this map set
     * @return An XML String containing the balance status
     *
     * @see #suspendBalancing(String, String)
     * @see #resumeBalancing(String, String)
     * @since 7.1.1
     */
    public String balanceStatus(String objectGridName, String mapSetName);
    
    /**
     * Re-enables a disabled container for placement.  A container may become  
     * disabled because of a falure to place a shard into into the container.<p>
     * 
     * Use the {@link #getDisabledForPlacement()} attribute to determine which containers are disabled.
     *
     * @param containerName
     *            The name of the container to re-enable.
     * @return Answers true if the container's status was changed from disabled to enabled, false if the container was 
     * already enabled for placement.
     * @since 8.6
     */
    public boolean enableForPlacement(String containerName);
    
    /**
     * Retrieves TabularData of containers which have been disabled because of the failure of shard placement operations.
     * 
     * The TabularData contains CompositeData with the following items, where each CompositeData represents a container:
     * <table>
     * <tr>
     * <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr> <td>Container</td><td>String</td><td>A container that has been disabled for placement</td> </tr>
     * </table>
     *
     * @return A TabularData of the names of disabled containers.
     * @since 8.6
     */
    public TabularData getDisabledForPlacement();

}
