// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.

package com.ibm.websphere.xsa;

import java.io.Externalizable;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import com.ibm.websphere.xs.rest.RestValueFactory;

/**
 * This object is used in the XC10 REST Gateway.   
 * It is a data wrapper for data to be accessed via XC10 REST Gateway.
 * With its wrapped data, it will be stored in an object grid's map entry, with a
 * String as the key of the map entry.
 * 
 * @deprecated Use the {@link com.ibm.websphere.xs.rest.RestValue} interface and the {@link RestValueFactory}.
 * @since 8.6, XC10 1.0.0.4
 */
public class RestValue implements com.ibm.websphere.xs.rest.RestValue, Externalizable {

	/**
	 * Contained value
	 */
	protected byte[] value=null;
	
	/**
	 * Content type of the contained value
	 * 
	 */
	protected String contentType=null;
	
	/**
	 * Flag to indicate if the value is compressed or not
	 * 
	 */
	protected boolean compressed=false;
	
	private static final long serialVersionUID = 1L; 
	

	/**
	 * Constructor
	 * 
	 * @since 8.6, XC10 1.0.0.4
	 */
	public RestValue() {
	}

	/**
	 * Constructor
	 * 
	 * @since 8.6, XC10 1.0.0.4
	 * 
	 * @param value
	 * 			A value to be wrapped by RestValue class.
	 * @param contentType
	 * 			Content type of the value.
	 */
	public RestValue(byte[] value, String contentType) {
		super();
		this.value = value;
		this.contentType = contentType;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
		+ ((contentType == null) ? 0 : contentType.hashCode());
		result = prime * result + Arrays.hashCode(value);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RestValue other = (RestValue) obj;
		if (contentType == null) {
			if (other.contentType != null)
				return false;
		} else if (!contentType.equals(other.contentType))
			return false;
		if (!Arrays.equals(value, other.value))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "RestValue [contentType=" + contentType + ", value="
		+ Arrays.toString(value) + "]";
	}

	/**
     * Return contained value
     * 
     * @since 8.6, XC10 1.0.0.4
     * @return 
     * 		An array of byte that represents the contained value
     */
	public byte[] getValue() {
		return value;
	}

	/**
	 * Set contained value
	 * 
	 * @since 8.6, XC10 1.0.0.4
	 * @param value
	 * 		An array of byte that represents the contained value
	 */
	public void setValue(byte[] value) {
		this.value = value;
	}

	/**
     * Return content type of the contained value
     * 
     * @since 8.6, XC10 1.0.0.4
     * @return
     * 		A content type
     */
	public String getContentType() {
		return contentType;
	}

	/**
	 * Set content type of the contained value
	 * 
	 * @since 8.6, XC10 1.0.0.4
	 * @param contentType
	 * 		A content type
	 */
	public void setContentType(String contentType) {
		this.contentType = contentType;
	}

	@Override
	public void readExternal(ObjectInput in) throws IOException,ClassNotFoundException {
		//1. read version
		byte ver = in.readByte();
		
		//2. read content-type
		byte ct = in.readByte();
		if (ct == 0)
			contentType = (String) in.readObject();
		else {
			contentType = getContentTypeString(ct);
			byte cs = in.readByte();
			String charSet=null;
			// 0 = read charset string
			// 1 = no charset specified
			// 2+ = mapped charset
			if (cs==0) {
				charSet = (String) in.readObject();
			} else if (cs!=1) {
				charSet = getCharsetString(cs);  
			}
			if (charSet != null) {
				contentType = contentType + "; charset="+charSet;
			}
		}
		
		//3. read compressed
		compressed=in.readBoolean();
		
		//4. read value len
		int length = in.readInt();
		
		//5. read value
		value = new byte[length];
		int bytesRead=0;
		while (bytesRead!=length)
			bytesRead+=in.read(value,bytesRead,length-bytesRead); 
	}

	@Override
	public void writeExternal(ObjectOutput out) throws IOException {
		//1. write out version
		out.writeByte(0); //ver0
		
		//2. write out contentType
		byte ct = (byte) getContentTypeByte(contentType);
		out.writeByte(ct);
		if (ct==0)
			out.writeObject(contentType);
		else {
			String charSet = getCharset(contentType);
			byte cs = (byte) getCharsetByte(charSet);
			out.writeByte(cs);
			if (cs==0)
				out.writeObject(charSet);
		}
		
		//3 write out if compressed
		out.writeBoolean(compressed);
		
		//4. write out value length
		out.writeInt(value.length);
		
		//5. write out value
		out.write(value,0,value.length);
	}

	static HashMap<String,Integer> contentTypeToInteger = new HashMap<String,Integer>();
	static HashMap<Integer,String> integerToContentType = new HashMap<Integer,String>();
	static {
		contentTypeToInteger.put("application/xml",1);
		contentTypeToInteger.put("application/octet-stream",2);
		contentTypeToInteger.put("application/json",3);
		contentTypeToInteger.put("application/javascript",4);
		contentTypeToInteger.put("application/pdf",5);
		contentTypeToInteger.put("application/zip",6);
		contentTypeToInteger.put("application/soap+xml",7);
		contentTypeToInteger.put("application/xhtml+xml",8);
		contentTypeToInteger.put("application/xml-dtd",9);
		contentTypeToInteger.put("message/http",10);
		contentTypeToInteger.put("text/css",11);
		contentTypeToInteger.put("text/csv",12);
		contentTypeToInteger.put("text/html",13);
		contentTypeToInteger.put("text/javascript",14);
		contentTypeToInteger.put("text/plain",15);
		contentTypeToInteger.put("text/xml",16);

		Iterator<String> it = contentTypeToInteger.keySet().iterator();
		while (it.hasNext()) {
			String ct = it.next();
			integerToContentType.put(contentTypeToInteger.get(ct),ct); 
		}
	}

	private static String getContentTypeString(int ct) {
		return integerToContentType.get(ct);
	}

	private static int getContentTypeByte(String ct) {
		int i = ct.indexOf(";");
		if (i!=-1)
			ct=ct.substring(0,i).trim();
		Integer intValue=contentTypeToInteger.get(ct);
		if (intValue==null){
			//if the content-type is unknown then per http://www.w3.org/Protocols/rfc1341/4_Content-Type.html it is application/octet-stream
			//adding so we don't blow up
			intValue=2;
		}
		return intValue;
	}

	private static String getCharset(String ct) {
		int i = ct.indexOf("charset=");
		if (i!=-1)
			return ct.substring(i+8).toLowerCase();
		return null;
	}

	static HashMap<String,Integer> charsetToInteger = new HashMap<String,Integer>();
	static HashMap<Integer,String> integerToCharset = new HashMap<Integer,String>();
	static {
		charsetToInteger.put("utf-8",2);
		charsetToInteger.put("utf8",3);
		charsetToInteger.put("iso-8859-1",4);
		charsetToInteger.put("iso8859_1",5);
		charsetToInteger.put("cp437",6);
		charsetToInteger.put("cp1252",7);
		charsetToInteger.put("cp850",8);
		charsetToInteger.put("utf-16",9);
		charsetToInteger.put("sjis",10);
		charsetToInteger.put("big5",11);
		charsetToInteger.put("ascii",12);
		charsetToInteger.put("windows-1252",13);
		charsetToInteger.put("euc-kr",14);
		charsetToInteger.put("shift_jis",15);
		charsetToInteger.put("iso-8859-2",16);
		charsetToInteger.put("iso-8859-3",17);
		charsetToInteger.put("iso-8859-4",18);
		charsetToInteger.put("iso-8859-5",19);
		charsetToInteger.put("macintosh",20);
		charsetToInteger.put("euc-jp",21);
		charsetToInteger.put("iso-2022-jp",22);
		charsetToInteger.put("iso-2022-kr",23);
		Iterator<String> it = charsetToInteger.keySet().iterator();
		while (it.hasNext()) {
			String cs = it.next();
			integerToCharset.put(charsetToInteger.get(cs),cs); 
		}
	}

	private static int getCharsetByte(String cs) {
		if (cs!=null)
			return charsetToInteger.get(cs);
		return 1;
	}

	private static String getCharsetString(int cs) {
		return integerToCharset.get(cs);
	}

	/**
	 * Indicate if the contained value is compressed
	 * 
	 * @since 8.6, XC10 1.0.0.4
	 * @return
	 * 		true if the contained value is compressed
	 */
	public boolean isCompressed() {
		return compressed;
	}

	/**
	 * Set indicator to indicate if the contained value is compressed
	 * 
	 * @param compressed
	 * 		A boolean value that indicate if the contained value is compressed
	 */
	public void setCompressed(boolean compressed) {
		this.compressed = compressed;
	}
}
